#!/usr/bin/env python2.7
# encoding: utf-8
'''
run_parser - Command Line Parser Class

@author:     Chris Fawcett, Marius Lindauer
        
@copyright:  2014 AClib. All rights reserved.
        
@license:    GPL

@contact:    lindauer@cs.uni-freiburg.de
'''

import os

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

class CMD_Parser(object):
    
    def __init__(self, program_license, program_version_message, aclib_root):
    
        default_config_file = os.path.join(aclib_root, "src", "data", "config.json")
        self.parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        run_group = self.parser.add_argument_group("Run")
        run_group.add_argument("-s", "--scenario", dest="scenario", default=None, help="scenario to run")
        run_group.add_argument("-c", "--configurator", dest="configurator", default=None, help="configurator to use")
        run_group.add_argument("-S", "--seed", type=int, dest="seed", default=1, help="random seed for the configurator")
        run_group.add_argument("-d", "--working_directory", dest="working_directory", default=".", help="working directory")
        run_group.add_argument("-p", "--print_cmd_only", dest="print_cmd_only", action="store_true", default=False, help="Don't run, just set up and print the command-line string")
        run_group.add_argument("-v", "--validate", dest="validate", action="store_true", default=False, help="Validate final incumbent")

        show_group = self.parser.add_argument_group("Show")
        show_group.add_argument("--show_scenarios", dest="show_scenarios", action="store_true", default=False, help="show all possible scenarios")
        
        misc_group = self.parser.add_argument_group("Misc")
        misc_group.add_argument("-C", "--config", dest="config_file", default=default_config_file, help="path to the AClib configuration file")
        self.parser.add_argument('-V', '--version', action='version', version=program_version_message)
        
    def parse(self):
        '''
            parses from sys.argv the command line arguments using self.parser
            and returns an ArgumentParser Object
        '''
        return self.parser.parse_args()
    
class CLUSTER_Parser(CMD_Parser):
    
    def __init__(self, program_license, program_version_message, aclib_root):
        
        CMD_Parser.__init__(self, program_license, program_version_message, aclib_root)
        
        cluster_group = self.parser.add_argument_group("Cluster Options")
        cluster_group.add_argument("-n", "--number_of_runs", dest="n_runs", default=1, type=int, help="number of independent runs")
        cluster_group.add_argument("-w", "--walltime", dest="walltime", default=100, type=int, help="walltime of cluster job [hours]")
        cluster_group.add_argument("--cluster", dest="cluster", default="meta", choices=["meta"], help="used cluster (has to be implemented!)")
        
        