#!/usr/local/bin/python2.7
# encoding: utf-8
'''
downloader -- downloads components of aclib

@author:     Marius Lindauer
        
@copyright:  2013 AClib. All rights reserved.
        
@license:    GPL

@contact:    lindauer@informatik.uni-freiburg.de
'''

import sys
import os
import hashlib

class Examiner(object):
    '''
        examines AClib folder:
        what is already installed 
        and what needs to be downloaded
    '''


    def __init__(self):
        '''
        Constructor
        '''
        src_path = os.path.dirname(sys.argv[0])
        self._aclib_path, _ = os.path.split(src_path)
        
    #===========================================================================
    # def examine_scenario(self, scenario_key, scenario_dict, problem_type, algo_types):
    #     '''
    #         examine whether all files of a scenario are already installed
    #         Returns:
    #             True: if all necessary files are found
    #             False: otherwise
    #     '''
    #     
    #     scenario_dir = os.path.join(self._aclib_path, "scenarios", problem_type, scenario_key)
    #     if not self.__check_dir(scenario_dir): return False
    #     
    #     scenario_file = os.path.join(scenario_dir, "scenario.txt")
    #     if not self.__check_file(scenario_file): return False
    #     
    #     if len(algo_types) > 1:
    #         algorithm_dir = os.path.join(self._aclib_path, "target_algorithms", "multi_problem", scenario_dict["algorithm"])
    #     else:
    #         algorithm_dir = os.path.join(self._aclib_path, "target_algorithms", algo_types[0], scenario_dict["algorithm"])
    #     if not self.__check_dir(algorithm_dir): return False
    #     
    #     pcs_file_name = os.path.basename(scenario_dict["pcs"])
    #     pcs_file = os.path.join(algorithm_dir, pcs_file_name)
    #     if not self.__check_file(pcs_file): return False
    #     
    #     instance_specific_dir = os.path.join(self._aclib_path, "instances", problem_type, "sets", scenario_dict["instances"])
    #     if not self.__check_dir(instance_specific_dir): return False
    #     
    #     training_file = os.path.join(instance_specific_dir, "training.txt")
    #     if not self.__check_file(training_file): return False
    #     
    #     test_file = os.path.join(instance_specific_dir, "test.txt")
    #     if not self.__check_file(test_file): return False
    #     
    #     test_file = os.path.join(instance_specific_dir, "README.md")
    #     if not self.__check_file(test_file): return False
    #===========================================================================
        
        # optional
        #=======================================================================
        # feature_file = os.path.join(instance_specific_dir, "features.txt")
        # if not self.__check_file(feature_file): return False
        #=======================================================================
            
        #----------------------------------------------------------- return True
            
    def examine_algorithm(self, algorithm_key, problem_type_list):
        '''
            checks whether algorithm is in target_algorithms/<algorithm_key>
            Returns:
                True: if all necessary files are found
                False: otherwise
        '''
        if len(problem_type_list) > 1:
            algorithm_dir = os.path.join(self._aclib_path, "target_algorithms", "multi_problem", algorithm_key)
            if not self.__check_dir(algorithm_dir): return False
            for ptype in problem_type_list:
                link_dir = os.path.join(self._aclib_path, "target_algorithms", ptype, algorithm_key)
                if not self.__check_link(link_dir): return False
        else:
            algorithm_dir = os.path.join(self._aclib_path, "target_algorithms",problem_type_list[0], algorithm_key)
            if not self.__check_dir(algorithm_dir): return False
        
        return True
    
    def examine_instances(self, instance_key, instance_dict):
        '''
            examines whether instances are in instances/<problem_type>/data/instance_key
            checks the hash of the instances
            assumption: all instances of a problem type are in the same archive
            Returns:
                True: if all necessary files are found
                False: otherwise
        '''
        instance_dir = os.path.join(self._aclib_path, "instances", instance_dict["problem_type"], "data", instance_key)
        if not self.__check_dir(instance_dir): return False
        
        #sys.stdout.write("check checksum of instances...\n")
        #sys.stdout.flush()
        print(instance_dir)
        checksum = self.__get_md5_hash(instance_dir)
        if instance_dict["md5"] != str(checksum):
            sys.stdout.write("fail\n")
            print("expected: %s \t encountered: %s" %(instance_dict["md5"], str(checksum)))
            return False
        sys.stdout.write("ok\n")
        sys.stdout.flush()
        return True
        
        
    def __get_md5_hash(self, dir_):
        '''
            computes the hash sum of all files in a directory
            Returns:
                checksum based on md5
        '''
        sys.stdout.write("Compute md5 hash of %s\n" %(dir_))
        files = []
        md5 = hashlib.md5()
        for dirpath, _, filenames in os.walk(dir_, topdown=True, followlinks=False):
            for file_ in filenames:
                file_path = os.path.join(dirpath, file_)
                files.append(file_path)
        
        files = sorted(files)
        for file_ in files:
            with open(file_, "rb") as f:
                for chunk in iter(lambda: f.read(81920), b''):
                    md5.update(chunk)
        return md5.hexdigest()

    def __check_file(self, file_):
        sys.stdout.write("check %s..." %(file_))
        sys.stdout.flush()
        if not os.path.isfile(file_):
            sys.stdout.write("fail\n")
            return False
        sys.stdout.write("ok\n")
        sys.stdout.flush()
        return True
        
    def __check_dir(self, dir_):
        sys.stdout.write("check %s..." %(dir_))
        sys.stdout.flush()
        if not os.path.isdir(dir_):
            sys.stdout.write("fail\n")
            return False
        sys.stdout.write("ok\n")
        sys.stdout.flush()
        return True
     
    def __check_link(self, dir_):
        sys.stdout.write("check %s..." %(dir_))
        sys.stdout.flush()
        if not os.path.islink(dir_):
            sys.stdout.write("fail\n")
            return False
        sys.stdout.write("ok\n")
        sys.stdout.flush()
        return True   
        
