'''
Created on 14.02.2014

@author: Marius Lindauer
'''

import json
import urllib2

class Updater(object):
    '''
        checker whether updates of aclib are available and print some stats
    '''

    def __init__(self, config_file):
        '''
            Constructor
        '''
        
        with open(config_file, "r") as fp:
            self.config = json.load(fp)
    
        self.__ONLINE_JSON = "http://aclib.net/data/config.json"
    
    def check_update(self):
        '''
            only method of class....
        '''
        req = urllib2.Request(self.__ONLINE_JSON)
        opener = urllib2.build_opener()
        fp = opener.open(req) 
        on_config = json.load(fp)
        fp.close()
            
        on_version = on_config.get("version")
        off_version = self.config.get("version")
        off_config = self.config
        
        if not on_version or not off_version or on_version > off_version: #TODO: Fix comparison
            print("Update available from %s to %s" %(str(off_version), str(on_version)))
        else:
            print("No Update available.")
            return False
        
        new_scenarios = set(on_config["scenarios"].keys()).difference(set(off_config["scenarios"].keys()))
        new_algorithms = set(on_config["algorithms"].keys()).difference(set(off_config["algorithms"].keys()))
        new_instance_sets = set(on_config["instances"].keys()).difference(set(off_config["instances"].keys()))
        
        if new_scenarios:
            print("New scenarios (%d): %s" %(len(new_scenarios), ",".join(new_scenarios)))
        if new_algorithms:
            print("New algorithms (%d): %s" %(len(new_algorithms),",".join(new_algorithms)))
        if new_instance_sets:
            print("New instance sets (%d): %s" %(len(new_instance_sets),",".join(new_instance_sets)))
            
        updated_instance_sets = []
        for old_inst_id in off_config["instances"].keys():
            old_hash = off_config["instances"][old_inst_id]["md5"]
            new_hash = on_config["instances"][old_inst_id]["md5"]
            if old_hash != new_hash:
                updated_instance_sets.append(old_inst_id)
        
        if updated_instance_sets:
            print("Updated instance sets (%d): %s" %(len(updated_instance_sets),",".join(updated_instance_sets)))
            
        